@@ -1,4 +1,4 @@
-from typing import Optional
+from typing import Callable, Dict, Optional
 from selenium import webdriver
 from selenium.common.exceptions import TimeoutException
 from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
@@ -10,14 +10,17 @@ from selenium.webdriver.support.ui import WebDriverWait
 from selenium.webdriver.support import expected_conditions as EC
 import time
 import random
+import requests
+
 
 class SelectBy(By):
     pass
 
+
 class ExploitBuilder:
     def __init__(
-        self, 
-        protocol: str, 
+        self,
+        protocol: str,
         hostname: str,
         **options
     ):
@@ -30,6 +33,8 @@ class ExploitBuilder:
         self.protocol = protocol
         self.hostname = hostname
         self.stealth = stealth
+        self.use_proxy = use_proxy
+        self.proxy_address = proxy_address
 
         additional_options = options.copy()
         additional_options.pop("use_proxy", None)
@@ -42,7 +47,8 @@ class ExploitBuilder:
             self._init_chrome(use_proxy, proxy_address, **additional_options)
 
     def _init_firefox(self, use_proxy, proxy_address, **options):
-        firefox_binary_path = options.get("firefox_binary_path", "/opt/firefox/firefox")
+        firefox_binary_path = options.get(
+            "firefox_binary_path", "/opt/firefox/firefox")
         binary = FirefoxBinary(firefox_binary_path)
         if use_proxy and proxy_address:
             proxy = Proxy({
@@ -54,10 +60,14 @@ class ExploitBuilder:
             proxy.add_to_capabilities(webdriver.DesiredCapabilities.FIREFOX)
             fp = webdriver.FirefoxProfile()
             fp.set_preference("network.proxy.type", 1)
-            fp.set_preference("network.proxy.http", proxy_address.split(":")[0])
-            fp.set_preference("network.proxy.http_port", int(proxy_address.split(":")[1]))
+            fp.set_preference("network.proxy.http",
+                              proxy_address.split(":")[0])
+            fp.set_preference("network.proxy.http_port",
+                              int(proxy_address.split(":")[1]))
             fp.set_preference('network.proxy.allow_hijacking_localhost', True)
             fp.update_preferences()
+            fp.accept_untrusted_certs = True
+            fp.assume_untrusted_cert_issuer = True
             self.driver = webdriver.Firefox(
                 firefox_binary=binary,
                 proxy=proxy,
@@ -67,15 +77,109 @@ class ExploitBuilder:
             self.driver = webdriver.Firefox(firefox_binary=binary)
 
     def _init_chrome(self, use_proxy, proxy_address, **options):
-        chromedriver_path = options.get("chromedriver_path", "/opt/chromedriver")
-        chrome_binary_path = options.get("chrome_binary_path", "/opt/google/chrome/google-chrome")
+        chromedriver_path = options.get(
+            "chromedriver_path", "/opt/chromedriver")
+        chrome_binary_path = options.get(
+            "chrome_binary_path", "/opt/google/chrome/google-chrome")
 
         options = Options()
         options.binary_location = chrome_binary_path
         if use_proxy and proxy_address:
             options.add_argument(f'--proxy-server={proxy_address}')
-        self.driver = webdriver.Chrome(chrome_options=options, executable_path=chromedriver_path)
+            options.add_argument('ignore-certificate-errors')
+        self.driver = webdriver.Chrome(
+            chrome_options=options, executable_path=chromedriver_path)
 
+    def api(self, req_method: Callable, endpoint: str, api_protocol: str = None, api_hostname: str = None, **kwargs) -> requests.Response:
+        """api
+        Wrapper for Python's Requests library, enabling automatic proxy usage and cookie sharing when interfacing with APIs. 
+        kwargs supports same options as Requests.
+
+        Parameters:
+            req_method (Callable): requests function to use (requests.get, requests.posts, etc.)
+            endpoint (str): the API endpoint to request
+            api_protocol (str): the protocol to use for the API. Effectively defaults to the same protocol used in the ExploitBuilder
+            api_hostname (str): the hostname to use for the API. Effectively defaults to the same hostname used in the ExploitBuilder
+            kwargs: interoperable with Python Requests (e.g. headers parameter will be passed into the requests call)
+        
+        Returns:
+        requests.Response
+        """
+        url = self._format(endpoint, api_protocol, api_hostname)
+        cookies = self._merge_cookies(**kwargs)
+        req_options = kwargs.copy()
+        req_options.pop("cookies", None)
+        if self.use_proxy:
+            proxy_url = self._format("/", "http", self.proxy_address)
+            req_options['proxies'] = {
+                'http': proxy_url,
+                'https': proxy_url
+            }
+        return req_method(url, cookies=cookies, verify=False, **req_options)
+
+    def api_get(self, endpoint: str, api_protocol: str = None, api_hostname: str = None, **kwargs) -> requests.Response:
+        """api_get
+        Wrapper for ExploitBuilder.api(requests.get, ...).
+
+        Parameters:
+            req_method (Callable): requests function to use (requests.get, requests.posts, etc.)
+            endpoint (str): the API endpoint to request
+            api_protocol (str): the protocol to use for the API. Effectively defaults to the same protocol used in the ExploitBuilder
+            api_hostname (str): the hostname to use for the API. Effectively defaults to the same hostname used in the ExploitBuilder
+            kwargs: interoperable with Python Requests (e.g. headers parameter will be passed into the requests call)
+        
+        Returns:
+        requests.Response
+        """
+        return self.api(requests.get, endpoint, api_protocol, api_hostname, **kwargs)
+
+    def api_post(self, endpoint: str, api_protocol: str = None, api_hostname: str = None, **kwargs) -> requests.Response:
+        """api_post
+        Wrapper for ExploitBuilder.api(requests.post, ...).
+
+        Parameters:
+            req_method (Callable): requests function to use (requests.get, requests.posts, etc.)
+            endpoint (str): the API endpoint to request
+            api_protocol (str): the protocol to use for the API. Effectively defaults to the same protocol used in the ExploitBuilder
+            api_hostname (str): the hostname to use for the API. Effectively defaults to the same hostname used in the ExploitBuilder
+            kwargs: interoperable with Python Requests (e.g. headers parameter will be passed into the requests call)
+        
+        Returns:
+        requests.Response
+        """
+        return self.api(requests.post, endpoint, api_protocol, api_hostname, **kwargs)
+
+    def api_put(self, endpoint: str, api_protocol: str = None, api_hostname: str = None, **kwargs) -> requests.Response:
+        """api_put
+        Wrapper for ExploitBuilder.api(requests.put, ...).
+
+        Parameters:
+            req_method (Callable): requests function to use (requests.get, requests.posts, etc.)
+            endpoint (str): the API endpoint to request
+            api_protocol (str): the protocol to use for the API. Effectively defaults to the same protocol used in the ExploitBuilder
+            api_hostname (str): the hostname to use for the API. Effectively defaults to the same hostname used in the ExploitBuilder
+            kwargs: interoperable with Python Requests (e.g. headers parameter will be passed into the requests call)
+
+        Returns:
+        requests.Response
+        """
+        return self.api(requests.put, endpoint, api_protocol, api_hostname, **kwargs)
+
+    def api_delete(self, endpoint: str, api_protocol: str = None, api_hostname: str = None, **kwargs) -> requests.Response:
+        """api_delete
+        Wrapper for ExploitBuilder.api(requests.delete, ...).
+
+        Parameters:
+            req_method (Callable): requests function to use (requests.get, requests.posts, etc.)
+            endpoint (str): the API endpoint to request
+            api_protocol (str): the protocol to use for the API. Effectively defaults to the same protocol used in the ExploitBuilder
+            api_hostname (str): the hostname to use for the API. Effectively defaults to the same hostname used in the ExploitBuilder
+            kwargs: interoperable with Python Requests (e.g. headers parameter will be passed into the requests call)
+
+        Returns:
+        requests.Response
+        """
+        return self.api(requests.delete, endpoint, api_protocol, api_hostname, **kwargs)
 
     def click(self, button: str, by: SelectBy = SelectBy.XPATH):
         """click
@@ -140,6 +244,18 @@ class ExploitBuilder:
             print(cookie)
         return self
 
+    def execute_script(self, script, *args):
+        """execute_script
+        Wrapper for Selenium's webdriver.execute_script().
+
+        Parameters:
+            script (str): The JavaScript to execute
+        
+        Returns:
+        Any: the return value of the JavaScript
+        """
+        return self.driver.execute_script(script, *args)
+
     def get(self, endpoint: str):
         """get
         Makes a GET request to the specified endpoint in the browser.
@@ -300,7 +416,7 @@ class ExploitBuilder:
         self.driver.find_element(By.XPATH, xpath).send_keys(Keys.ENTER)
         return self
 
-    def set_cookie(self, name: str, value: str, path: str ="/", secure: bool=False):
+    def set_cookie(self, name: str, value: str, path: str = "/", secure: bool = False):
         """set_cookie
         Set a cookie in the browser.
 
@@ -385,7 +501,7 @@ class ExploitBuilder:
         element.send_keys(entry)
         return self
 
-    def wait_for_stealth(self, entry: str =""):
+    def wait_for_stealth(self, entry: str = ""):
         """wait_for_stealth
         Waits for a period of time based on the length of the entry.
 
@@ -410,7 +526,8 @@ class ExploitBuilder:
         Returns:
         bool: whether or not the alert fired"""
         try:
-            WebDriverWait(self.driver, timeout).until(EC.alert_is_present(), "Timeout waiting for alert.")
+            WebDriverWait(self.driver, timeout).until(
+                EC.alert_is_present(), "Timeout waiting for alert.")
             alert = self.driver.switch_to.alert
             alert_text = alert.text
             alert.accept()
@@ -420,14 +537,38 @@ class ExploitBuilder:
         except TimeoutException:
             return False
 
-    def _format(self, endpoint: str) -> str:
+    def _format(self, endpoint: str, protocol: str = None, hostname: str = None) -> str:
         """_format
         Formats an endpoint into a usable URL with the exploit builder.
 
         Parameters:
             endpoint (str): The endpoint to format.
+            protocol (str): The protocol to format (defaults to the protocol in the ExploitBuilder instance)
+            hostname (str): The hostname to format (defaults to the hostname in the ExploitBuilder instance)
 
         Returns:
         str: The formatted URL"""
+        if not protocol:
+            protocol = self.protocol
+        if not hostname:
+            hostname = self.hostname
+        return f"{protocol}://{hostname}{endpoint}"
 
-        return f"{self.protocol}://{self.hostname}{endpoint}"
+    def _merge_cookies(self, **kwargs) -> Dict[str, str]:
+        """_merge_cookies
+        Helper function used to merge ExploitBuilder instance cookies with supplied cookies.
+
+        Parameters:
+            kwargs
+        
+        Returns:
+        Dict[str, str]: The final list of cookies, ready for use in requests
+        """
+        tmp_cookies = self.get_cookies()
+        arg_cookies = kwargs.get("cookies", {})
+        cookies = {}
+        for cookie in tmp_cookies:
+            cookies[cookie["name"]] = cookie["value"]
+        for cookie in arg_cookies:
+            cookies[cookie] = arg_cookies[cookie]
+        return cookies
